/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "main.h"
#include <windows.h>

/*
 * Maximum number of parameters associated with an option,
 * including the option name itself.
 */
#define MAX_PARMS 5


/* Connections parameters */
struct connections
{
  char config_file[MAX_PATH]; 	/* Name of the config file */
  char log_path[MAX_PATH];	/* Path to Logfile */
  int connect_status;		/* 0=Not Connected 1=Connecting 2=Connected */
  int auto_connect;		/* true=AutoConnect at startup */
  int ask_pass;			/* true=Ask for Passphrase before starting */
  int failed_psw;		/* 1=OpenVPN failed because of wrong psw */
  int failed_psw_attempts;	/* # of failed attempts maid to enter psw */
  HANDLE exit_event;
  HANDLE hProcess;
  HANDLE hStdOut;
  HANDLE hStdIn;
  HWND hwndStatus;		/* Handle to Status Dialog Window */
};

/* All options used within OpenVPN GUI */
struct options
{
  /* Array of configs to autostart */
  const char *auto_connect[MAX_CONFIGS];
  int auto_connect_nopass[MAX_CONFIGS];

  /* Connection parameters */
  struct connections cnn[MAX_CONFIGS];  /* Connection structure */
  int num_configs;			/* Number of configs */

  int oldversion;			/* 1=OpenVPN version below 2.0-beta6 */
  char connect_string[100];		/* String to look for to report connected */
  int psw_attempts;			/* Number of psw attemps to allow */
  HWND hWnd;				/* Main Window Handle */
  HINSTANCE hInstance;

  /* Registry values */
  char exe_path[MAX_PATH];
  char config_dir[MAX_PATH];
  char ext_string[16];
  char log_dir[MAX_PATH];
  char priority_string[64];
  char append_string[2];
  char log_viewer[MAX_PATH];
  char editor[MAX_PATH];
  char allow_edit[2];
  char allow_service[2];
  char psw_attempts_string[2];

  /* Proxy Settings */
  int proxy_source;			/* 0=OpenVPN config, 1=IE, 2=Manual */
  int proxy_type;			/* 0=HTTP, 1=SOCKS */
  int proxy_http_auth;			/* 0=Auth Disabled, 1=Auth Enabled */
  char proxy_http_address[100];		/* HTTP Proxy Address */
  char proxy_http_port[6];		/* HTTP Proxy Port */
  char proxy_socks_address[100];	/* SOCKS Proxy Address */
  char proxy_socks_port[6];		/* SOCKS Proxy Address */
  char proxy_authfile[100];		/* Path to proxy auth file */ 
};

#define streq(x, y) (!strcmp((x), (y)))
void init_options (struct options *o);
int Createargcargv(struct options* options, char* command_line);
void parse_argv (struct options* options, int argc, char *argv[]);
static int add_option (struct options *options, int i, char *p[]);
LONG GetRegistryValue(HKEY regkey, char *name, char *data, DWORD len);
int SetRegistryValue(HKEY regkey, char *name, char *data);

