/*
 *  OpenVPN-GUI -- A Windows GUI for OpenVPN.
 *
 *  Copyright (C) 2004 Mathias Sundman <mathias@nilings.se>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program (see the file COPYING included with this
 *  distribution); if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include "tray.h"
#include "main.h"
#include "options.h"
#include "openvpn-gui-res.h"

//POPUP MENU
HMENU hMenu;		// menu handle
HMENU hMenuConnect;	// menu handle
HMENU hMenuDisconnect;	// menu handle
HMENU hMenuStatus;	// menu handle
HMENU hMenuViewLog;	// menu handle
HMENU hMenuEditConfig;	// menu handle
HMENU hMenuService;	// menu handle
HMENU hMenuPassphrase;	// menu handle
HWND  hwnd;     	// main window handle

NOTIFYICONDATA ni;
int i;
int connected_config;

// Mouse clicks on tray
void OnNotifyTray(LPARAM lParam)
{
  POINT pt;				// point structure
  extern struct options o;
  // Right click, show the menu
  switch(lParam) {
    case WM_RBUTTONDOWN: 
      GetCursorPos(&pt);		// get the cursors position
      SetForegroundWindow(hwnd);	// set the foreground window
      TrackPopupMenu(hMenu,TPM_RIGHTALIGN,pt.x,pt.y,0,hwnd,NULL);// track the popup
      break;

    case WM_LBUTTONDOWN:
      break;

    case WM_LBUTTONDBLCLK:
      /* Open Status window if only one connection is running */
      connected_config = -1;
      for (i=0; i < o.num_configs; i++)
        {
          if(o.cnn[i].connect_status != 0)
            {
              if (connected_config == -1)
                {
                  connected_config = i;
                }
              else
                {
                  connected_config = -1;
                  break;
                }
            }
        }
      if (connected_config != -1)
        ShowWindow(o.cnn[connected_config].hwndStatus, SW_SHOW); 
      SetForegroundWindow(o.cnn[connected_config].hwndStatus);
      break;
  }
  PostMessage(hwnd,WM_NULL,0,0);// post a null message
}


void OnDestroyTray()
{
  //Destroy popup menu
  DestroyMenu(hMenu);
	
  //Remove Icon from Tray
  Shell_NotifyIcon(NIM_DELETE, &ni);
}


//Initialize tray
void TRAY_Init(HWND hWnd)
{
    hwnd = hWnd;
    hMenu=CreatePopupMenu();
    hMenuConnect=CreatePopupMenu();
    hMenuDisconnect=CreatePopupMenu();
    hMenuStatus=CreatePopupMenu();
    hMenuViewLog=CreatePopupMenu();
    hMenuEditConfig=CreatePopupMenu();
    hMenuPassphrase=CreatePopupMenu();
    hMenuService=CreatePopupMenu();
    CreateItemList(hMenu); 
    LoadAppIcon();
} 


//Create menu items
void CreateItemList(HMENU hMenu)
{
  extern struct options o;
  extern char ext_string[];
  extern char allow_edit[2];
  extern char allow_service[2];
  char conn_name[200];
  int i;
  
  // CREATE POPUP MENU ( on the fly )
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuConnect,IDM_TEXT_CONNECT); 
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuDisconnect,IDM_TEXT_DISCONNECT);
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuStatus,IDM_TEXT_STATUS);
  AppendMenu(hMenu,MF_SEPARATOR,0,0);
  if (o.allow_service[0]=='1') {
    AppendMenu(hMenu,MF_POPUP,(UINT) hMenuService,IDM_TEXT_SERVICE);
    AppendMenu(hMenu,MF_SEPARATOR,0,0);
  }
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuViewLog,IDM_TEXT_VIEWLOG);
  if (o.allow_edit[0]=='1')
    AppendMenu(hMenu,MF_POPUP,(UINT) hMenuEditConfig,IDM_TEXT_EDITCONFIG);

#ifndef DISABLE_CHANGE_PASSWORD
  AppendMenu(hMenu,MF_POPUP,(UINT) hMenuPassphrase,IDM_TEXT_PASSPHRASE);
#endif
  AppendMenu(hMenu,MF_SEPARATOR,0,0);
  AppendMenu(hMenu,MF_STRING ,IDM_PROXY,IDM_TEXT_PROXY);
  AppendMenu(hMenu,MF_SEPARATOR,0,0);
  AppendMenu(hMenu,MF_STRING ,IDM_ABOUT,IDM_TEXT_ABOUT);
  AppendMenu(hMenu,MF_STRING ,IDM_CLOSE,IDM_TEXT_CLOSE);

  /* Create sub-menus */
  for (i=0; i < o.num_configs; i++) {

    /* Cut of extention from config filename. */
    strncpy(conn_name, o.cnn[i].config_file, sizeof(conn_name));
    conn_name[strlen(conn_name) - (strlen(o.ext_string)+1)]=0;

    AppendMenu(hMenuConnect,MF_STRING, IDM_CONNECTMENU+i, conn_name);  
    AppendMenu(hMenuDisconnect,MF_STRING, IDM_DISCONNECTMENU+i, conn_name);  
    AppendMenu(hMenuStatus,MF_STRING, IDM_STATUSMENU+i, conn_name);  
    AppendMenu(hMenuViewLog,MF_STRING, IDM_VIEWLOGMENU+i, conn_name);  
    AppendMenu(hMenuEditConfig,MF_STRING, IDM_EDITMENU+i, conn_name);  
    AppendMenu(hMenuPassphrase,MF_STRING, IDM_PASSPHRASEMENU+i, conn_name);  
    SetMenuStatus(i, false); 
  }

  /* Create Service menu */
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_START, IDM_TEXT_SERVICE_START);  
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_STOP, IDM_TEXT_SERVICE_STOP);  
  AppendMenu(hMenuService,MF_STRING, IDM_SERVICE_RESTART, IDM_TEXT_SERVICE_RESTART);  

}


BOOL LoadAppIcon()
{
  // Load icon from resource
  HICON hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(APP_ICON), 
                                                IMAGE_ICON, 0, 0, LR_DEFAULTCOLOR);
  if (hIcon) {
    SendMessage(hwnd, WM_SETICON, (WPARAM) (ICON_SMALL), (LPARAM) (hIcon));
    SendMessage(hwnd, WM_SETICON, (WPARAM) (ICON_BIG), (LPARAM) (hIcon));  //ALT+TAB icon
    return TRUE;
  }
  return FALSE;
}


void ShowTrayIcon()
{
  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  lstrcpyn(ni.szTip, MSG_TIP, sizeof(ni.szTip)); 
  ni.hWnd = hwnd;
  ni.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON; // We want to use icon, tip, and callback message
  ni.uCallbackMessage = WM_NOTIFYICONTRAY;      // Our custom callback message (WM_APP + 1)
    
     
  //Load selected icon
  ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_DISCONNECTED));	

  Shell_NotifyIcon(NIM_ADD, &ni);       
}

/* SetTrayIcon(int connected)
 * connected=0 -> DisConnected
 * connected=1 -> Connecting
 * connected=2 -> Connected
 */
void SetTrayIcon(int connected)
{
  char msg[500];
  char conn_name[200];
  int i, first_conn;
  extern struct options o;

  ni.cbSize = sizeof(ni);
  ni.uID = 0;
  ni.hWnd = hwnd;
  ni.uFlags = NIF_MESSAGE | NIF_TIP | NIF_ICON; // We want to use icon, tip, and callback message
  ni.uCallbackMessage = WM_NOTIFYICONTRAY;      // Our custom callback message (WM_APP + 1)
    
  strncpy(msg, MSG_TIP, sizeof(ni.szTip)); 

  first_conn=1;
  for (i=0; i < o.num_configs; i++)
    {
      if(o.cnn[i].connect_status == 2)
        {
          /* Cut of extention from config filename. */
          strncpy(conn_name, o.cnn[i].config_file, sizeof(conn_name));
          conn_name[strlen(conn_name) - (strlen(o.ext_string)+1)]=0;

          /* Append connection name to Icon Tip Msg */
          if (first_conn)
            strncat(msg, "\nConnected to: ", sizeof(msg) - strlen(msg) - 1);
          else
            strncat(msg, ", ", sizeof(msg) - strlen(msg) - 1);
          strncat(msg, conn_name, sizeof(msg) - strlen(msg) - 1);
          first_conn=0;
        }
    }
  strncpy(ni.szTip, msg, sizeof(ni.szTip)); 
     
  //Load selected icon
  if (connected==2) 
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_CONNECTED));	
  else if (connected==1)
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_CONNECTING));	
  else if (connected==0)
    ni.hIcon = (HICON)LoadIcon((HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE), 
                                                       MAKEINTRESOURCE(APP_ICON_DISCONNECTED));	

  Shell_NotifyIcon(NIM_MODIFY, &ni);       
}


void SetMenuStatus (int config, int bCheck)
{
  /* bCheck values:
   * 0 - Not Connected
   * 1 - Connecting
   * 2 - Connected
   * 3 - Disconnecting
   */

  int iState;

  iState = ((bCheck == 2) || (bCheck == 3)) ? MF_CHECKED : MF_UNCHECKED ;
  CheckMenuItem (hMenuConnect, IDM_CONNECTMENU + config, iState) ;
  CheckMenuItem (hMenuDisconnect, IDM_DISCONNECTMENU + config, iState) ;
  CheckMenuItem (hMenuStatus, IDM_STATUSMENU + config, iState) ;
  
  if (bCheck == 0) {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_ENABLED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuStatus, IDM_STATUSMENU + config, MF_GRAYED);
  }
  else if (bCheck == 1) {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_ENABLED);
    EnableMenuItem(hMenuStatus, IDM_STATUSMENU + config, MF_ENABLED);
  }
  else if (bCheck == 2) {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_ENABLED);
    EnableMenuItem(hMenuStatus, IDM_STATUSMENU + config, MF_ENABLED);
  }
  else if (bCheck == 3) {
    EnableMenuItem(hMenuConnect, IDM_CONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuDisconnect, IDM_DISCONNECTMENU + config, MF_GRAYED);
    EnableMenuItem(hMenuStatus, IDM_STATUSMENU + config, MF_ENABLED);
  }
}

void SetServiceMenu(BOOL bCheck)
{
  /* 1 = Service is started.
   * 0 = Service is not started."
   */
  if (bCheck) {
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_ENABLED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_ENABLED);
  }
  else {
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_ENABLED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_GRAYED);
  }
}


void SetServiceMenuDisabled()
{
    EnableMenuItem(hMenuService, IDM_SERVICE_START, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_STOP, MF_GRAYED);
    EnableMenuItem(hMenuService, IDM_SERVICE_RESTART, MF_GRAYED);
}


